import {
  world,
  BlockPermutation,
  system,
  Block,
  BlockComponentTickEvent,
  EntityEquippableComponent,
  EquipmentSlot,
  ItemEnchantableComponent,
  EnchantmentTypes,
  EnchantmentSlot,
} from "@minecraft/server";
const maxBlockHandlePerTick = 2500;
/**
 *
 * @param {str} str
 * @param {string} appendText
 */
function specialAppend(str, appendText) {
  if (appendText.length == 0) return str;
  if (str.length > 1) str += "_";
  str += appendText;
  return str;
}
/**
 *  @type {((arg,expectedTypeId)=>void)[]}
 */
var queue = new Array();
/**
 * @type {Set<string>}
 */
var lastCoordinates = new Set();
world.beforeEvents.worldInitialize.subscribe((worldEvent) => {
  const same_block_on_up = "mcenters_better_glasses:same_block_on_up";
  const same_block_on_down = "mcenters_better_glasses:same_block_on_down";
  const same_block_on_east = "mcenters_better_glasses:same_block_on_east";
  const same_block_on_west = "mcenters_better_glasses:same_block_on_west";
  const same_block_on_south = "mcenters_better_glasses:same_block_on_south";
  const same_block_on_north = "mcenters_better_glasses:same_block_on_north";
  /**
   *
   * @param {Block} arg
   * @param {string} expectedTypeId
   * @returns
   */
  var func2 = (arg, expectedTypeId) => {
    const block = arg;
    if (!block.isValid()) return;
    if (block.typeId != expectedTypeId) return;
    const typeId = block.typeId;
    const records = {};
    const old_records = block.permutation.getAllStates();
    records[same_block_on_up] = block.above()?.typeId == typeId;
    records[same_block_on_down] = block.below()?.typeId == typeId;
    records[same_block_on_north] = block.north()?.typeId == typeId;
    records[same_block_on_east] = block.east()?.typeId == typeId;
    records[same_block_on_west] = block.west()?.typeId == typeId;
    records[same_block_on_south] = block.south()?.typeId == typeId;

    if (
      old_records[same_block_on_up] != undefined &&
      old_records[same_block_on_down] != undefined &&
      old_records[same_block_on_north] != undefined &&
      old_records[same_block_on_east] != undefined &&
      old_records[same_block_on_west] != undefined &&
      old_records[same_block_on_south] != undefined &&
      (records[same_block_on_up] != old_records[same_block_on_up] ||
        records[same_block_on_down] != old_records[same_block_on_down] ||
        records[same_block_on_north] != old_records[same_block_on_north] ||
        records[same_block_on_east] != old_records[same_block_on_east] ||
        records[same_block_on_west] != old_records[same_block_on_west] ||
        records[same_block_on_south] != old_records[same_block_on_south])
    ) {
      block.setPermutation(BlockPermutation.resolve(typeId, records));
    }
  };
  /**
   *
   * @param {Block} block
   */
  var func = (block) => {
    try{
    if (
      block.isValid()
      // &&
      // !lastCoordinates.has(JSON.stringify(block.location))
    ) {
      // lastCoordinates.add(JSON.stringify(block.location));
      queue.push(func2.bind(undefined, block, block.typeId));
    }
  }
  catch(err){}
  };
  worldEvent.blockComponentRegistry.registerCustomComponent(
    "mcenters_better_glasses:connected_glasses",
    {
      onPlace: (arg) => {
        try {
          const block = arg.block;
          const typeId = block.typeId;

          const aboveBlock = block.above();
          const belowBlock = block.below();
          const eastBlock = block.east();
          const westBlock = block.west();
          const northBlock = block.north();
          const southBlock = block.south();
          if (aboveBlock?.typeId == typeId) func(aboveBlock);
          if (belowBlock?.typeId == typeId) func(belowBlock);
          if (eastBlock?.typeId == typeId) func(eastBlock);
          if (westBlock?.typeId == typeId) func(westBlock);
          if (northBlock?.typeId == typeId) func(northBlock);
          if (southBlock?.typeId == typeId) func(southBlock);

          func(block);
        } catch (err) {}
      },
      onPlayerDestroy: (arg) => {
        try {
          const block = arg.block;
          const typeId = arg.destroyedBlockPermutation.type.id;

          const aboveBlock = block.above();
          const belowBlock = block.below();
          const eastBlock = block.east();
          const westBlock = block.west();
          const northBlock = block.north();
          const southBlock = block.south();
          if (aboveBlock?.typeId == typeId) func(aboveBlock);
          if (belowBlock?.typeId == typeId) func(belowBlock);
          if (eastBlock?.typeId == typeId) func(eastBlock);
          if (westBlock?.typeId == typeId) func(westBlock);
          if (northBlock?.typeId == typeId) func(northBlock);
          if (southBlock?.typeId == typeId) func(southBlock);
        } catch (err) {}
      },
      onTick: (arg) => {
        try {
          if (queue.length >= maxBlockHandlePerTick) return;
          func(arg.block);
        } catch (err) {}
      },
      beforeOnPlayerPlace: (arg) => {
        if(!world.gameRules.doTileDrops) return;
        const player = arg.player;
        if (player == undefined || !player.isValid() || !arg.block.isValid())
          return;

        const block = arg.block;
        if(block.hasTag("mcenters_better_glasses:skip_before_player_destroy"))return;
        /**
         * @type {EntityEquippableComponent|undefined}
         */
        const equipment = player.getComponent(
          EntityEquippableComponent.componentId
        );
        if (equipment == undefined || !equipment.isValid()) return;
        var item = equipment.getEquipment(EquipmentSlot.Mainhand);
        if (item == undefined) return;

        /**
         * @type {ItemEnchantableComponent|undefined}
         */
        var enchantmentComponent = item.getComponent(
          ItemEnchantableComponent.componentId
        );
        if (
          enchantmentComponent == undefined ||
          !enchantmentComponent.isValid() ||
          !enchantmentComponent.hasEnchantment("silk_touch")
        )
          return;
        arg.dimension.spawnItem(block.typeId, block.location);
      },
    }
  );
});
function* HandleQueue() {
  while (true) {
    try {
      if (queue.length == 0 && lastCoordinates.size != 0)
        lastCoordinates.clear();
      if (queue.length == 0) yield;
      var entry = queue.pop();
      if (entry == undefined) continue;
      entry();
    } catch (err) {}
    yield;
  }
}
system.runJob(HandleQueue());
